#ifndef DIRLIST_H
#define DIRLIST_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

/*---------------------------*
 | dirlist.h                 |
 |---------------------------|
 | creates directory tree    |
 *---------------------------*/

#include <sys/stat.h>
#include <sys/types.h>

/*---------------------------------- TYPEDEFS -------------------------------*/

/* protection values */
typedef enum { NONE = 0, READ = 1, WRITE = 2, EXECUTE = 4 } SECURITY_FLAGS;

/* Dir Entry Structure */
typedef struct {
  SECURITY_FLAGS user;   /* user security flags */
  SECURITY_FLAGS group;  /* group security flags */
  SECURITY_FLAGS others; /* others security flags */

  int opened;   /* opened directory flag */
  int readable; /* readable directory flag */
  
  uid_t UID;    /* User ID */
  gid_t GID;    /* Group ID */
  
  time_t changetime;
  time_t modiftime;  /* Last modification date */
  time_t accesstime;

  off_t size;        /* file size (in bytes) */

  char *pathname; /* full pathname */
  char *name; /* name only */
} TDirEntry;
  
/* Node for Directory Tree */
struct DirNode {
  TDirEntry dir;   /* dir values */

  struct DirNode *parent; /* parent directory */

  int nb_children; /* number of children (current level) */
  struct DirNode **child_list; /* children list */
};

/* general node type */
typedef struct DirNode TDirNode; /* general node type */


/*----------------------------------- FUNCTIONS -----------------------------*/


/********************************
 * CreateRootNode               *
 ********************************
 * Create the first entry       *
 ********************************/

extern TDirNode *CreateRootNode(void);

/********************************
 * CreateSubDirNode             *
 ********************************
 * Create a new node for a      *
 * sub-directory                *
 ********************************/

extern TDirNode *CreateSubdirNode(struct stat *statbuf, TDirNode *parent, char *name);

/********************************
 * MakeDirOneLevel              *
 ********************************
 * finds all the subdirs of     *
 * the directory argument       *
 ********************************/

extern int MakeDirOneLevel(TDirNode *actualnode);

/********************************
 * DeleteSubdirNode             *
 ********************************
 * delete recurively all        *
 * subdirs of current node      *
 ********************************/

extern void DeleteSubdirNode(TDirNode *parent);

/********************************
 * DeleteRootDir                *
 ********************************
 * Delete first entry           *
 ********************************/      

extern void DeleteRootDir(TDirNode *node);

#endif



